@@ -109,6 +109,7 @@ public:
     bool onInsertsIsGlobalWriteLockExclusive = false;
 
     bool onRenameCollectionCalled = false;
+    bool onRenameCollectionDropTarget = false;
     repl::OpTime renameOpTime = {Timestamp(Seconds(100), 1U), 1LL};
 
 private:
@@ -175,6 +176,7 @@ repl::OpTime OpObserverMock::onRenameCollection(OperationContext* opCtx,
     OpObserverNoop::onRenameCollection(
         opCtx, fromCollection, toCollection, uuid, dropTarget, dropTargetUUID, stayTemp);
     onRenameCollectionCalled = true;
+    onRenameCollectionDropTarget = dropTarget;
     return renameOpTime;
 }
 
@@ -675,6 +677,7 @@ TEST_F(RenameCollectionTest, RenameCollectionMakesTargetCollectionDropPendingIfD
                                                              << " missing after successful rename";
 
     ASSERT_TRUE(_opObserver->onRenameCollectionCalled);
+    ASSERT_TRUE(_opObserver->onRenameCollectionDropTarget);
 
     auto renameOpTime = _opObserver->renameOpTime;
     ASSERT_GREATER_THAN(renameOpTime, repl::OpTime());
@@ -726,6 +729,21 @@ TEST_F(RenameCollectionTest,
         << " renamed to drop-pending collection after successful rename";
 }
 
+TEST_F(RenameCollectionTest,
+       RenameCollectionOverridesDropTargetIfTargetCollectionIsMissingAndDropTargetIsTrue) {
+    _createCollectionWithUUID(_opCtx.get(), _sourceNss);
+    RenameCollectionOptions options;
+    options.dropTarget = true;
+    ASSERT_OK(renameCollection(_opCtx.get(), _sourceNss, _targetNss, options));
+    ASSERT_FALSE(_collectionExists(_opCtx.get(), _sourceNss))
+        << "source collection " << _sourceNss << " still exists after successful rename";
+    ASSERT_TRUE(_collectionExists(_opCtx.get(), _targetNss)) << "target collection " << _targetNss
+                                                             << " missing after successful rename";
+
+    ASSERT_TRUE(_opObserver->onRenameCollectionCalled);
+    ASSERT_FALSE(_opObserver->onRenameCollectionDropTarget);
+}
+
 TEST_F(RenameCollectionTest, RenameCollectionForApplyOpsRejectsRenameOpTimeIfWritesAreReplicated) {
     ASSERT_TRUE(_opCtx->writesAreReplicated());
 
