@@ -50,6 +50,7 @@
 #include "mongo/db/audit.h"
 #include "mongo/db/auth/authorization_manager.h"
 #include "mongo/db/auth/authorization_manager_global.h"
+#include "mongo/db/auth/sasl_options.h"
 #include "mongo/db/catalog/collection.h"
 #include "mongo/db/catalog/create_collection.h"
 #include "mongo/db/catalog/database.h"
@@ -163,6 +164,7 @@
 #include "mongo/util/quick_exit.h"
 #include "mongo/util/ramlog.h"
 #include "mongo/util/scopeguard.h"
+#include "mongo/util/sequence_util.h"
 #include "mongo/util/signal_handlers.h"
 #include "mongo/util/stacktrace.h"
 #include "mongo/util/startup_test.h"
@@ -170,6 +172,10 @@
 #include "mongo/util/time_support.h"
 #include "mongo/util/version.h"
 
+#ifdef MONGO_CONFIG_SSL
+#include "mongo/util/net/ssl_options.h"
+#endif
+
 #if !defined(_WIN32)
 #include <sys/file.h>
 #endif
@@ -772,6 +778,18 @@ ExitCode _initAndListen(int listenPort) {
 
     logMongodStartupWarnings(storageGlobalParams, serverGlobalParams, serviceContext);
 
+#ifdef MONGO_CONFIG_SSL
+    if (sslGlobalParams.sslAllowInvalidCertificates &&
+        ((serverGlobalParams.clusterAuthMode.load() == ServerGlobalParams::ClusterAuthMode_x509) ||
+         sequenceContains(saslGlobalParams.authenticationMechanisms, "MONGODB-X509"))) {
+        log() << "** WARNING: While invalid X509 certificates may be used to" << startupWarningsLog;
+        log() << "**          connect to this server, they will not be considered"
+              << startupWarningsLog;
+        log() << "**          permissible for authentication." << startupWarningsLog;
+        log() << startupWarningsLog;
+    }
+#endif
+
     {
         std::stringstream ss;
         ss << endl;
@@ -914,10 +932,6 @@ ExitCode _initAndListen(int listenPort) {
 
     SessionCatalog::create(serviceContext);
 
-    // Set up the periodic runner for background job execution
-    auto runner = makePeriodicRunner();
-    serviceContext->setPeriodicRunner(std::move(runner));
-
     // This function may take the global lock.
     auto shardingInitialized =
         uassertStatusOK(ShardingState::get(startupOpCtx.get())
@@ -991,6 +1005,11 @@ ExitCode _initAndListen(int listenPort) {
 
     PeriodicTask::startRunningPeriodicTasks();
 
+    // Set up the periodic runner for background job execution
+    auto runner = makePeriodicRunner();
+    runner->startup().transitional_ignore();
+    serviceContext->setPeriodicRunner(std::move(runner));
+
     SessionKiller::set(serviceContext,
                        std::make_shared<SessionKiller>(serviceContext, killSessionsLocal));
 
@@ -998,7 +1017,6 @@ ExitCode _initAndListen(int listenPort) {
 
     // Set up the logical session cache
     LogicalSessionCacheServer kind = LogicalSessionCacheServer::kStandalone;
-
     if (serverGlobalParams.clusterRole == ClusterRole::ShardServer) {
         kind = LogicalSessionCacheServer::kSharded;
     } else if (serverGlobalParams.clusterRole == ClusterRole::ConfigServer) {
@@ -1007,14 +1025,8 @@ ExitCode _initAndListen(int listenPort) {
         kind = LogicalSessionCacheServer::kReplicaSet;
     }
 
-    // Shards and config servers set up the LogicalSessionCache when they go through sharding
-    // state initialization
-    if (kind == LogicalSessionCacheServer::kStandalone ||
-        kind == LogicalSessionCacheServer::kReplicaSet) {
-        invariant(LogicalSessionCache::get(startupOpCtx.get()) == nullptr);
-        auto sessionCache = makeLogicalSessionCacheD(serviceContext, kind);
-        LogicalSessionCache::set(serviceContext, std::move(sessionCache));
-    }
+    auto sessionCache = makeLogicalSessionCacheD(serviceContext, kind);
+    LogicalSessionCache::set(serviceContext, std::move(sessionCache));
 
     // MessageServer::run will return when exit code closes its socket and we don't need the
     // operation context anymore
@@ -1032,9 +1044,6 @@ ExitCode _initAndListen(int listenPort) {
         return EXIT_NET_ERROR;
     }
 
-    // Start the periodic runner
-    uassertStatusOK(serviceContext->getPeriodicRunner()->startup());
-
     serviceContext->notifyStartupComplete();
 
 #ifndef _WIN32
