@@ -33,6 +33,7 @@
 #include "mongo/db/repl/bgsync.h"
 
 #include "mongo/base/counter.h"
+#include "mongo/base/string_data.h"
 #include "mongo/bson/util/bson_extract.h"
 #include "mongo/client/connection_pool.h"
 #include "mongo/db/auth/authorization_session.h"
@@ -43,17 +44,20 @@
 #include "mongo/db/repl/data_replicator_external_state_impl.h"
 #include "mongo/db/repl/oplog.h"
 #include "mongo/db/repl/oplog_interface_local.h"
+#include "mongo/db/repl/oplog_interface_remote.h"
 #include "mongo/db/repl/replication_coordinator_global.h"
 #include "mongo/db/repl/replication_coordinator_impl.h"
+#include "mongo/db/repl/replication_process.h"
 #include "mongo/db/repl/rollback_source_impl.h"
 #include "mongo/db/repl/rs_rollback.h"
+#include "mongo/db/repl/rs_rollback_no_uuid.h"
 #include "mongo/db/repl/rs_sync.h"
 #include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/s/shard_identity_rollback_notifier.h"
+#include "mongo/db/server_parameters.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/rpc/metadata/repl_set_metadata.h"
 #include "mongo/stdx/memory.h"
-#include "mongo/util/fail_point_service.h"
 #include "mongo/util/log.h"
 #include "mongo/util/mongoutils/str.h"
 #include "mongo/util/time_support.h"
@@ -69,6 +73,27 @@ const char kHashFieldName[] = "h";
 const int kSleepToAllowBatchingMillis = 2;
 const int kSmallBatchLimitBytes = 40000;
 const Milliseconds kRollbackOplogSocketTimeout(10 * 60 * 1000);
+// 16MB max batch size / 12 byte min doc size * 10 (for good measure) = defaultBatchSize to use.
+const auto defaultBatchSize = (16 * 1024 * 1024) / 12 * 10;
+
+// The batchSize to use for the find/getMore queries called by the OplogFetcher
+MONGO_EXPORT_STARTUP_SERVER_PARAMETER(bgSyncOplogFetcherBatchSize, int, defaultBatchSize);
+
+// The batchSize to use for the find/getMore queries called by the rollback common point resolver.
+// A batchSize of 0 means that the 'find' and 'getMore' commands will be given no batchSize.
+// We set the default to 2000 to prevent the sync source from having to read too much data at once,
+// and reduce the chance of a socket timeout.
+// We choose 2000 for (10 minute timeout) * (60 sec / min) * (50 MB / second) / (16 MB / document).
+constexpr int defaultRollbackBatchSize = 2000;
+MONGO_EXPORT_SERVER_PARAMETER(rollbackRemoteOplogQueryBatchSize, int, defaultRollbackBatchSize)
+    ->withValidator([](const auto& potentialNewValue) {
+        if (potentialNewValue < 0) {
+            return Status(ErrorCodes::BadValue,
+                          "rollbackRemoteOplogQueryBatchSize cannot be negative.");
+        }
+
+        return Status::OK();
+    });
 
 /**
  * Extends DataReplicatorExternalStateImpl to be member state aware.
@@ -131,29 +156,31 @@ static ServerStatusMetricField<Counter64> displayBufferMaxSize("repl.buffer.maxS
 
 BackgroundSync::BackgroundSync(
     ReplicationCoordinatorExternalState* replicationCoordinatorExternalState,
+    ReplicationProcess* replicationProcess,
     std::unique_ptr<OplogBuffer> oplogBuffer)
     : _oplogBuffer(std::move(oplogBuffer)),
       _replCoord(getGlobalReplicationCoordinator()),
-      _replicationCoordinatorExternalState(replicationCoordinatorExternalState) {
+      _replicationCoordinatorExternalState(replicationCoordinatorExternalState),
+      _replicationProcess(replicationProcess) {
     // Update "repl.buffer.maxSizeBytes" server status metric to reflect the current oplog buffer's
     // max size.
     bufferMaxSizeGauge.increment(_oplogBuffer->getMaxSize() - bufferMaxSizeGauge.get());
 }
 
-void BackgroundSync::startup(OperationContext* txn) {
-    _oplogBuffer->startup(txn);
+void BackgroundSync::startup(OperationContext* opCtx) {
+    _oplogBuffer->startup(opCtx);
 
     invariant(!_producerThread);
     _producerThread.reset(new stdx::thread(stdx::bind(&BackgroundSync::_run, this)));
 }
 
-void BackgroundSync::shutdown(OperationContext* txn) {
+void BackgroundSync::shutdown(OperationContext* opCtx) {
     stdx::lock_guard<stdx::mutex> lock(_mutex);
 
     // Clear the buffer. This unblocks the OplogFetcher if it is blocked with a full queue, but
     // ensures that it won't add anything. It will also unblock the OpApplier pipeline if it is
     // waiting for an operation to be past the slaveDelay point.
-    clearBuffer(txn);
+    clearBuffer(opCtx);
     _state = ProducerState::Stopped;
 
     if (_syncSourceResolver) {
@@ -164,12 +191,16 @@ void BackgroundSync::shutdown(OperationContext* txn) {
         _oplogFetcher->shutdown();
     }
 
+    if (_rollback) {
+        _rollback->shutdown();
+    }
+
     _inShutdown = true;
 }
 
-void BackgroundSync::join(OperationContext* txn) {
+void BackgroundSync::join(OperationContext* opCtx) {
     _producerThread->join();
-    _oplogBuffer->shutdown(txn);
+    _oplogBuffer->shutdown(opCtx);
 }
 
 bool BackgroundSync::inShutdown() const {
@@ -183,7 +214,7 @@ bool BackgroundSync::_inShutdown_inlock() const {
 
 void BackgroundSync::_run() {
     Client::initThread("rsBackgroundSync");
-    AuthorizationSession::get(cc())->grantInternalAuthorization(&cc());
+    AuthorizationSession::get(cc())->grantInternalAuthorization();
 
     while (!inShutdown()) {
         try {
@@ -208,15 +239,9 @@ void BackgroundSync::_runProducer() {
         return;
     }
 
-    // TODO(spencer): Use a condition variable to await loading a config.
-    // TODO(siyuan): Control bgsync with producer state.
-    if (_replCoord->getMemberState().startup()) {
-        // Wait for a config to be loaded
-        sleepsecs(1);
-        return;
-    }
-
-    invariant(!_replCoord->getMemberState().rollback());
+    auto memberState = _replCoord->getMemberState();
+    invariant(!memberState.rollback());
+    invariant(!memberState.startup());
 
     // We need to wait until initial sync has started.
     if (_replCoord->getMyLastAppliedOpTime().isNull()) {
@@ -252,6 +277,10 @@ void BackgroundSync::_produce() {
 
     // this oplog reader does not do a handshake because we don't want the server it's syncing
     // from to track how far it has synced
+    HostAndPort oldSource;
+    OpTime lastOpTimeFetched;
+    HostAndPort source;
+    SyncSourceResolverResponse syncSourceResp;
     {
         stdx::unique_lock<stdx::mutex> lock(_mutex);
         if (_lastOpTimeFetched.isNull()) {
@@ -265,19 +294,16 @@ void BackgroundSync::_produce() {
         if (_state != ProducerState::Running) {
             return;
         }
+
+        oldSource = _syncSourceHost;
     }
 
     // find a target to sync from the last optime fetched
-    OpTime lastOpTimeFetched;
-    HostAndPort source;
-    HostAndPort oldSource = _syncSourceHost;
-    SyncSourceResolverResponse syncSourceResp;
     {
         OpTime minValidSaved;
         {
             auto opCtx = cc().makeOperationContext();
-            auto storageInterface = StorageInterface::get(opCtx.get());
-            minValidSaved = storageInterface->getMinValid(opCtx.get());
+            minValidSaved = _replicationProcess->getConsistencyMarkers()->getMinValid(opCtx.get());
         }
         stdx::lock_guard<stdx::mutex> lock(_mutex);
         if (_state != ProducerState::Running) {
@@ -317,7 +343,7 @@ void BackgroundSync::_produce() {
             log() << "Our newest OpTime : " << lastOpTimeFetched;
             log() << "Earliest OpTime available is " << syncSourceResp.earliestOpTimeSeen
                   << " from " << syncSourceResp.getSyncSource();
-            _replCoord->abortCatchupIfNeeded();
+            _replCoord->abortCatchupIfNeeded().transitional_ignore();
             return;
         }
 
@@ -329,6 +355,10 @@ void BackgroundSync::_produce() {
         // Mark yourself as too stale.
         _tooStale = true;
 
+        // Need to take global X lock to transition out of SECONDARY.
+        auto opCtx = cc().makeOperationContext();
+        Lock::GlobalWrite globalWriteLock(opCtx.get());
+
         error() << "too stale to catch up -- entering maintenance mode";
         log() << "Our newest OpTime : " << lastOpTimeFetched;
         log() << "Earliest OpTime available is " << syncSourceResp.earliestOpTimeSeen;
@@ -339,10 +369,10 @@ void BackgroundSync::_produce() {
         if (!status.isOK()) {
             warning() << "Failed to transition into maintenance mode: " << status;
         }
-        bool worked = _replCoord->setFollowerMode(MemberState::RS_RECOVERING);
-        if (!worked) {
+        status = _replCoord->setFollowerMode(MemberState::RS_RECOVERING);
+        if (!status.isOK()) {
             warning() << "Failed to transition into " << MemberState(MemberState::RS_RECOVERING)
-                      << ". Current state: " << _replCoord->getMemberState();
+                      << ". Current state: " << _replCoord->getMemberState() << causedBy(status);
         }
         return;
     } else if (syncSourceResp.isOK() && !syncSourceResp.getSyncSource().empty()) {
@@ -403,9 +433,9 @@ void BackgroundSync::_produce() {
     // before the OplogWriter gets a chance to append to the oplog.
     {
         auto opCtx = cc().makeOperationContext();
-        auto storageInterface = StorageInterface::get(opCtx.get());
-        if (storageInterface->getAppliedThrough(opCtx.get()).isNull()) {
-            storageInterface->setAppliedThrough(opCtx.get(), _replCoord->getMyLastAppliedOpTime());
+        if (_replicationProcess->getConsistencyMarkers()->getAppliedThrough(opCtx.get()).isNull()) {
+            _replicationProcess->getConsistencyMarkers()->setAppliedThrough(
+                opCtx.get(), _replCoord->getMyLastAppliedOpTime());
         }
     }
 
@@ -415,19 +445,18 @@ void BackgroundSync::_produce() {
         _replCoord, _replicationCoordinatorExternalState, this);
     OplogFetcher* oplogFetcher;
     try {
-        auto onOplogFetcherShutdownCallbackFn =
-            [&fetcherReturnStatus](const Status& status, const OpTimeWithHash& lastFetched) {
-                fetcherReturnStatus = status;
-            };
+        auto onOplogFetcherShutdownCallbackFn = [&fetcherReturnStatus](const Status& status) {
+            fetcherReturnStatus = status;
+        };
         // The construction of OplogFetcher has to be outside bgsync mutex, because it calls
         // replication coordinator.
         auto oplogFetcherPtr = stdx::make_unique<OplogFetcher>(
             _replicationCoordinatorExternalState->getTaskExecutor(),
             OpTimeWithHash(lastHashFetched, lastOpTimeFetched),
             source,
-            NamespaceString(rsOplogName),
+            NamespaceString::kRsOplogNamespace,
             _replCoord->getConfig(),
-            _replicationCoordinatorExternalState->getOplogFetcherSteadyStateMaxFetcherRestarts(),
+            _replicationCoordinatorExternalState->getOplogFetcherMaxFetcherRestarts(),
             syncSourceResp.rbid,
             true /* requireFresherSyncSource */,
             &dataReplicatorExternalState,
@@ -436,20 +465,21 @@ void BackgroundSync::_produce() {
                        stdx::placeholders::_1,
                        stdx::placeholders::_2,
                        stdx::placeholders::_3),
-            onOplogFetcherShutdownCallbackFn);
+            onOplogFetcherShutdownCallbackFn,
+            bgSyncOplogFetcherBatchSize);
         stdx::lock_guard<stdx::mutex> lock(_mutex);
         if (_state != ProducerState::Running) {
             return;
         }
         _oplogFetcher = std::move(oplogFetcherPtr);
         oplogFetcher = _oplogFetcher.get();
-    } catch (const mongo::DBException& ex) {
+    } catch (const mongo::DBException&) {
         fassertFailedWithStatus(34440, exceptionToStatus());
     }
 
     const auto logLevel = Command::testCommandsEnabled ? 0 : 1;
     LOG(logLevel) << "scheduling fetcher to read remote oplog on " << _syncSourceHost
-                  << " starting at " << oplogFetcher->getCommandObject_forTest()["filter"];
+                  << " starting at " << oplogFetcher->getFindQuery_forTest()["filter"];
     auto scheduleStatus = oplogFetcher->startup();
     if (!scheduleStatus.isOK()) {
         warning() << "unable to schedule fetcher to read remote oplog on " << source << ": "
@@ -478,73 +508,10 @@ void BackgroundSync::_produce() {
         // if it can't return a matching oplog start from the last fetch oplog ts field.
         return;
     } else if (fetcherReturnStatus.code() == ErrorCodes::OplogStartMissing) {
-        if (_replCoord->getMemberState().primary()) {
-            warning() << "Rollback situation detected in catch-up mode. Aborting catch-up mode.";
-            _replCoord->abortCatchupIfNeeded();
-            return;
-        }
-
-        // Rollback is a synchronous operation that uses the task executor and may not be
-        // executed inside the fetcher callback.
-        const int messagingPortTags = 0;
-        ConnectionPool connectionPool(messagingPortTags);
-        std::unique_ptr<ConnectionPool::ConnectionPtr> connection;
-        auto getConnection = [&connection, &connectionPool, source]() -> DBClientBase* {
-            if (!connection.get()) {
-                connection.reset(new ConnectionPool::ConnectionPtr(
-                    &connectionPool, source, Date_t::now(), kRollbackOplogSocketTimeout));
-            };
-            return connection->get();
-        };
-
-        {
-            stdx::lock_guard<stdx::mutex> lock(_mutex);
-            lastOpTimeFetched = _lastOpTimeFetched;
-        }
-
-        log() << "Starting rollback due to " << redact(fetcherReturnStatus);
-
-        // TODO: change this to call into the Applier directly to block until the applier is
-        // drained.
-        //
-        // Wait till all buffered oplog entries have drained and been applied.
-        auto lastApplied = _replCoord->getMyLastAppliedOpTime();
-        if (lastApplied != lastOpTimeFetched) {
-            log() << "Waiting for all operations from " << lastApplied << " until "
-                  << lastOpTimeFetched << " to be applied before starting rollback.";
-            while (lastOpTimeFetched > (lastApplied = _replCoord->getMyLastAppliedOpTime())) {
-                sleepmillis(10);
-                if (getState() != ProducerState::Running) {
-                    return;
-                }
-            }
-        }
-
-        if (MONGO_FAIL_POINT(rollbackHangBeforeStart)) {
-            // This log output is used in js tests so please leave it.
-            log() << "rollback - rollbackHangBeforeStart fail point "
-                     "enabled. Blocking until fail point is disabled.";
-            while (MONGO_FAIL_POINT(rollbackHangBeforeStart) && !inShutdown()) {
-                mongo::sleepsecs(1);
-            }
-        }
-
-        {
-            auto opCtx = cc().makeOperationContext();
-            OplogInterfaceLocal localOplog(opCtx.get(), rsOplogName);
-            RollbackSourceImpl rollbackSource(getConnection, source, rsOplogName);
-            auto storageInterface = StorageInterface::get(opCtx.get());
-            rollback(opCtx.get(),
-                     localOplog,
-                     rollbackSource,
-                     syncSourceResp.rbid,
-                     _replCoord,
-                     storageInterface);
-        }
-
-        // Reset the producer to clear the sync source and the last optime fetched.
-        stop(true);
-        startProducerIfStopped();
+        auto opCtx = cc().makeOperationContext();
+        auto storageInterface = StorageInterface::get(opCtx.get());
+        _runRollback(
+            opCtx.get(), fetcherReturnStatus, source, syncSourceResp.rbid, storageInterface);
     } else if (fetcherReturnStatus == ErrorCodes::InvalidBSON) {
         Seconds blacklistDuration(60);
         warning() << "Fetcher got invalid BSON while querying oplog. Blacklisting sync source "
@@ -566,10 +533,10 @@ Status BackgroundSync::_enqueueDocuments(Fetcher::Documents::const_iterator begi
         return Status::OK();  // Nothing to do.
     }
 
-    auto txn = cc().makeOperationContext();
+    auto opCtx = cc().makeOperationContext();
 
     // Wait for enough space.
-    _oplogBuffer->waitForSpace(txn.get(), info.toApplyDocumentBytes);
+    _oplogBuffer->waitForSpace(opCtx.get(), info.toApplyDocumentBytes);
 
     {
         // Don't add more to the buffer if we are in shutdown. Continue holding the lock until we
@@ -586,7 +553,7 @@ Status BackgroundSync::_enqueueDocuments(Fetcher::Documents::const_iterator begi
         }
 
         // Buffer docs for later application.
-        _oplogBuffer->pushAllNonBlocking(txn.get(), begin, end);
+        _oplogBuffer->pushAllNonBlocking(opCtx.get(), begin, end);
 
         // Update last fetched info.
         _lastFetchedHash = info.lastDocument.value;
@@ -611,8 +578,8 @@ Status BackgroundSync::_enqueueDocuments(Fetcher::Documents::const_iterator begi
     return Status::OK();
 }
 
-bool BackgroundSync::peek(OperationContext* txn, BSONObj* op) {
-    return _oplogBuffer->peek(txn, op);
+bool BackgroundSync::peek(OperationContext* opCtx, BSONObj* op) {
+    return _oplogBuffer->peek(opCtx, op);
 }
 
 void BackgroundSync::waitForMore() {
@@ -620,11 +587,11 @@ void BackgroundSync::waitForMore() {
     _oplogBuffer->waitForData(Seconds(1));
 }
 
-void BackgroundSync::consume(OperationContext* txn) {
+void BackgroundSync::consume(OperationContext* opCtx) {
     // this is just to get the op off the queue, it's been peeked at
     // and queued for application already
     BSONObj op;
-    if (_oplogBuffer->tryPop(txn, &op)) {
+    if (_oplogBuffer->tryPop(opCtx, &op)) {
         bufferCountGauge.decrement(1);
         bufferSizeGauge.decrement(getSize(op));
     } else {
@@ -635,6 +602,144 @@ void BackgroundSync::consume(OperationContext* txn) {
     }
 }
 
+void BackgroundSync::_runRollback(OperationContext* opCtx,
+                                  const Status& fetcherReturnStatus,
+                                  const HostAndPort& source,
+                                  int requiredRBID,
+                                  StorageInterface* storageInterface) {
+    if (_replCoord->getMemberState().primary()) {
+        warning() << "Rollback situation detected in catch-up mode. Aborting catch-up mode.";
+        _replCoord->abortCatchupIfNeeded().transitional_ignore();
+        return;
+    }
+
+    // Rollback is a synchronous operation that uses the task executor and may not be
+    // executed inside the fetcher callback.
+
+    OpTime lastOpTimeFetched;
+    {
+        stdx::lock_guard<stdx::mutex> lock(_mutex);
+        lastOpTimeFetched = _lastOpTimeFetched;
+    }
+
+    log() << "Starting rollback due to " << redact(fetcherReturnStatus);
+    log() << "Replication commit point: " << _replCoord->getLastCommittedOpTime();
+
+    // TODO: change this to call into the Applier directly to block until the applier is
+    // drained.
+    //
+    // Wait till all buffered oplog entries have drained and been applied.
+    auto lastApplied = _replCoord->getMyLastAppliedOpTime();
+    if (lastApplied != lastOpTimeFetched) {
+        log() << "Waiting for all operations from " << lastApplied << " until " << lastOpTimeFetched
+              << " to be applied before starting rollback.";
+        while (lastOpTimeFetched > (lastApplied = _replCoord->getMyLastAppliedOpTime())) {
+            sleepmillis(10);
+            if (getState() != ProducerState::Running) {
+                return;
+            }
+        }
+    }
+
+    if (MONGO_FAIL_POINT(rollbackHangBeforeStart)) {
+        // This log output is used in js tests so please leave it.
+        log() << "rollback - rollbackHangBeforeStart fail point "
+                 "enabled. Blocking until fail point is disabled.";
+        while (MONGO_FAIL_POINT(rollbackHangBeforeStart) && !inShutdown()) {
+            mongo::sleepsecs(1);
+        }
+    }
+
+    OplogInterfaceLocal localOplog(opCtx, NamespaceString::kRsOplogNamespace.ns());
+
+    const int messagingPortTags = 0;
+    ConnectionPool connectionPool(messagingPortTags);
+    std::unique_ptr<ConnectionPool::ConnectionPtr> connection;
+    auto getConnection = [&connection, &connectionPool, source]() -> DBClientBase* {
+        if (!connection.get()) {
+            connection.reset(new ConnectionPool::ConnectionPtr(
+                &connectionPool, source, Date_t::now(), kRollbackOplogSocketTimeout));
+        };
+        return connection->get();
+    };
+
+    // Run a rollback algorithm that either uses UUIDs or does not use UUIDs depending on
+    // the FCV. Since collection UUIDs were only added in 3.6, the 3.4 rollback algorithm
+    // remains in place to maintain backwards compatibility.
+    if (serverGlobalParams.featureCompatibility.getVersion() ==
+        ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo36) {
+        // If the user is fully upgraded to FCV 3.6, use the "rollbackViaRefetch" method.
+        log() << "Rollback using the 'rollbackViaRefetch' method because UUID support "
+                 "is feature compatible with featureCompatibilityVersion 3.6.";
+        _fallBackOnRollbackViaRefetch(
+            opCtx, source, requiredRBID, &localOplog, true, getConnection);
+    } else {
+        // If the user is either fully downgraded to FCV 3.4, downgrading to FCV 3.4,
+        // or upgrading to FCV 3.6, use the "rollbackViaRefetchNoUUID" method.
+        log() << "Rollback using the 'rollbackViaRefetchNoUUID' method because UUID "
+                 "support is not feature compatible with featureCompatibilityVersion 3.4";
+        _fallBackOnRollbackViaRefetch(
+            opCtx, source, requiredRBID, &localOplog, false, getConnection);
+    }
+
+    // Reset the producer to clear the sync source and the last optime fetched.
+    stop(true);
+    startProducerIfStopped();
+}
+
+void BackgroundSync::_runRollbackViaRecoverToCheckpoint(
+    OperationContext* opCtx,
+    const HostAndPort& source,
+    OplogInterface* localOplog,
+    StorageInterface* storageInterface,
+    OplogInterfaceRemote::GetConnectionFn getConnection) {
+
+    OplogInterfaceRemote remoteOplog(getConnection,
+                                     NamespaceString::kRsOplogNamespace.ns(),
+                                     rollbackRemoteOplogQueryBatchSize.load());
+
+    {
+        stdx::lock_guard<stdx::mutex> lock(_mutex);
+        if (_state != ProducerState::Running) {
+            return;
+        }
+    }
+
+    fassertFailedNoTrace(40651);
+
+    _rollback = stdx::make_unique<RollbackImpl>(
+        localOplog, &remoteOplog, storageInterface, _replicationProcess, _replCoord);
+
+    log() << "Scheduling rollback (sync source: " << source << ")";
+    auto status = _rollback->runRollback(opCtx);
+    if (status.isOK()) {
+        log() << "Rollback successful.";
+    } else {
+        warning() << "Rollback failed with error: " << status;
+    }
+}
+
+void BackgroundSync::_fallBackOnRollbackViaRefetch(
+    OperationContext* opCtx,
+    const HostAndPort& source,
+    int requiredRBID,
+    OplogInterface* localOplog,
+    bool useUUID,
+    OplogInterfaceRemote::GetConnectionFn getConnection) {
+
+    RollbackSourceImpl rollbackSource(getConnection,
+                                      source,
+                                      NamespaceString::kRsOplogNamespace.ns(),
+                                      rollbackRemoteOplogQueryBatchSize.load());
+
+    if (useUUID) {
+        rollback(opCtx, *localOplog, rollbackSource, requiredRBID, _replCoord, _replicationProcess);
+    } else {
+        rollbackNoUUID(
+            opCtx, *localOplog, rollbackSource, requiredRBID, _replCoord, _replicationProcess);
+    }
+}
+
 HostAndPort BackgroundSync::getSyncTarget() const {
     stdx::unique_lock<stdx::mutex> lock(_mutex);
     return _syncSourceHost;
@@ -665,10 +770,10 @@ void BackgroundSync::stop(bool resetLastFetchedOptime) {
     }
 }
 
-void BackgroundSync::start(OperationContext* txn) {
+void BackgroundSync::start(OperationContext* opCtx) {
     OpTimeWithHash lastAppliedOpTimeWithHash;
     do {
-        lastAppliedOpTimeWithHash = _readLastAppliedOpTimeWithHash(txn);
+        lastAppliedOpTimeWithHash = _readLastAppliedOpTimeWithHash(opCtx);
         stdx::lock_guard<stdx::mutex> lk(_mutex);
         // Double check the state after acquiring the mutex.
         if (_state != ProducerState::Starting) {
@@ -693,38 +798,40 @@ void BackgroundSync::start(OperationContext* txn) {
     LOG(1) << "bgsync fetch queue set to: " << _lastOpTimeFetched << " " << _lastFetchedHash;
 }
 
-void BackgroundSync::clearBuffer(OperationContext* txn) {
-    _oplogBuffer->clear(txn);
+void BackgroundSync::clearBuffer(OperationContext* opCtx) {
+    _oplogBuffer->clear(opCtx);
     const auto count = bufferCountGauge.get();
     bufferCountGauge.decrement(count);
     const auto size = bufferSizeGauge.get();
     bufferSizeGauge.decrement(size);
 }
 
-OpTimeWithHash BackgroundSync::_readLastAppliedOpTimeWithHash(OperationContext* txn) {
+OpTimeWithHash BackgroundSync::_readLastAppliedOpTimeWithHash(OperationContext* opCtx) {
     BSONObj oplogEntry;
     try {
-        MONGO_WRITE_CONFLICT_RETRY_LOOP_BEGIN {
-            ScopedTransaction transaction(txn, MODE_IX);
-            Lock::DBLock lk(txn->lockState(), "local", MODE_X);
-            bool success = Helpers::getLast(txn, rsOplogName.c_str(), oplogEntry);
-            if (!success) {
-                // This can happen when we are to do an initial sync.  lastHash will be set
-                // after the initial sync is complete.
-                return OpTimeWithHash(0);
-            }
+        bool success = writeConflictRetry(
+            opCtx, "readLastAppliedHash", NamespaceString::kRsOplogNamespace.ns(), [&] {
+                Lock::DBLock lk(opCtx, "local", MODE_X);
+                return Helpers::getLast(
+                    opCtx, NamespaceString::kRsOplogNamespace.ns().c_str(), oplogEntry);
+            });
+
+        if (!success) {
+            // This can happen when we are to do an initial sync.  lastHash will be set
+            // after the initial sync is complete.
+            return OpTimeWithHash(0);
         }
-        MONGO_WRITE_CONFLICT_RETRY_LOOP_END(txn, "readLastAppliedHash", rsOplogName);
     } catch (const DBException& ex) {
-        severe() << "Problem reading " << rsOplogName << ": " << redact(ex);
+        severe() << "Problem reading " << NamespaceString::kRsOplogNamespace.ns() << ": "
+                 << redact(ex);
         fassertFailed(18904);
     }
     long long hash;
     auto status = bsonExtractIntegerField(oplogEntry, kHashFieldName, &hash);
     if (!status.isOK()) {
-        severe() << "Most recent entry in " << rsOplogName << " is missing or has invalid \""
-                 << kHashFieldName << "\" field. Oplog entry: " << redact(oplogEntry) << ": "
-                 << redact(status);
+        severe() << "Most recent entry in " << NamespaceString::kRsOplogNamespace.ns()
+                 << " is missing or has invalid \"" << kHashFieldName
+                 << "\" field. Oplog entry: " << redact(oplogEntry) << ": " << redact(status);
         fassertFailed(18902);
     }
     OplogEntry parsedEntry(oplogEntry);
@@ -748,8 +855,8 @@ bool BackgroundSync::shouldStopFetching() const {
     return false;
 }
 
-void BackgroundSync::pushTestOpToBuffer(OperationContext* txn, const BSONObj& op) {
-    _oplogBuffer->push(txn, op);
+void BackgroundSync::pushTestOpToBuffer(OperationContext* opCtx, const BSONObj& op) {
+    _oplogBuffer->push(opCtx, op);
     bufferCountGauge.increment();
     bufferSizeGauge.increment(op.objsize());
 }
