@@ -51,7 +51,8 @@ namespace {
 using UniqueLock = stdx::unique_lock<stdx::mutex>;
 }  // namespace
 
-class SnapshotThread;
+class DropPendingCollectionReaper;
+class ReplicationProcess;
 class StorageInterface;
 class NoopWriter;
 
@@ -60,52 +61,51 @@ class ReplicationCoordinatorExternalStateImpl final : public ReplicationCoordina
     MONGO_DISALLOW_COPYING(ReplicationCoordinatorExternalStateImpl);
 
 public:
-    ReplicationCoordinatorExternalStateImpl(StorageInterface* storageInterface);
+    ReplicationCoordinatorExternalStateImpl(
+        ServiceContext* service,
+        DropPendingCollectionReaper* dropPendingCollectionReaper,
+        StorageInterface* storageInterface,
+        ReplicationProcess* replicationProcess);
     virtual ~ReplicationCoordinatorExternalStateImpl();
     virtual void startThreads(const ReplSettings& settings) override;
-    virtual void startInitialSync(OnInitialSyncFinishedFn finished) override;
-    virtual void startSteadyStateReplication(OperationContext* txn,
+    virtual void startSteadyStateReplication(OperationContext* opCtx,
                                              ReplicationCoordinator* replCoord) override;
-    virtual void stopDataReplication(OperationContext* txn) override;
-    virtual void runOnInitialSyncThread(stdx::function<void(OperationContext* txn)> run) override;
+    virtual void stopDataReplication(OperationContext* opCtx) override;
 
-    virtual bool isInitialSyncFlagSet(OperationContext* txn) override;
+    virtual bool isInitialSyncFlagSet(OperationContext* opCtx) override;
 
-    virtual void startMasterSlave(OperationContext* txn);
-    virtual void shutdown(OperationContext* txn);
+    virtual void startMasterSlave(OperationContext* opCtx);
+    virtual void shutdown(OperationContext* opCtx);
     virtual executor::TaskExecutor* getTaskExecutor() const override;
     virtual OldThreadPool* getDbWorkThreadPool() const override;
-    virtual Status runRepairOnLocalDB(OperationContext* txn) override;
-    virtual Status initializeReplSetStorage(OperationContext* txn, const BSONObj& config);
-    void onDrainComplete(OperationContext* txn) override;
-    OpTime onTransitionToPrimary(OperationContext* txn, bool isV1ElectionProtocol) override;
+    virtual Status runRepairOnLocalDB(OperationContext* opCtx) override;
+    virtual Status initializeReplSetStorage(OperationContext* opCtx, const BSONObj& config);
+    virtual void waitForAllEarlierOplogWritesToBeVisible(OperationContext* opCtx);
+    void onDrainComplete(OperationContext* opCtx) override;
+    OpTime onTransitionToPrimary(OperationContext* opCtx, bool isV1ElectionProtocol) override;
     virtual void forwardSlaveProgress();
-    virtual OID ensureMe(OperationContext* txn);
-    virtual Status createLocalLastVoteCollection(OperationContext* txn);
-    virtual bool isSelf(const HostAndPort& host, ServiceContext* ctx);
-    virtual StatusWith<BSONObj> loadLocalConfigDocument(OperationContext* txn);
-    virtual Status storeLocalConfigDocument(OperationContext* txn, const BSONObj& config);
-    virtual StatusWith<LastVote> loadLocalLastVoteDocument(OperationContext* txn);
-    virtual Status storeLocalLastVoteDocument(OperationContext* txn, const LastVote& lastVote);
-    virtual void setGlobalTimestamp(const Timestamp& newTime);
-    virtual StatusWith<OpTime> loadLastOpTime(OperationContext* txn);
-    virtual void cleanUpLastApplyBatch(OperationContext* txn);
-    virtual HostAndPort getClientHostAndPort(const OperationContext* txn);
+    virtual OID ensureMe(OperationContext* opCtx);
+    virtual bool isSelf(const HostAndPort& host, ServiceContext* service);
+    virtual StatusWith<BSONObj> loadLocalConfigDocument(OperationContext* opCtx);
+    virtual Status storeLocalConfigDocument(OperationContext* opCtx, const BSONObj& config);
+    virtual StatusWith<LastVote> loadLocalLastVoteDocument(OperationContext* opCtx);
+    virtual Status storeLocalLastVoteDocument(OperationContext* opCtx, const LastVote& lastVote);
+    virtual void setGlobalTimestamp(ServiceContext* service, const Timestamp& newTime);
+    virtual StatusWith<OpTime> loadLastOpTime(OperationContext* opCtx);
+    virtual HostAndPort getClientHostAndPort(const OperationContext* opCtx);
     virtual void closeConnections();
-    virtual void killAllUserOperations(OperationContext* txn);
+    virtual void killAllUserOperations(OperationContext* opCtx);
     virtual void shardingOnStepDownHook();
     virtual void signalApplierToChooseNewSyncSource();
     virtual void stopProducer();
     virtual void startProducerIfStopped();
     void dropAllSnapshots() final;
-    void updateCommittedSnapshot(SnapshotName newCommitPoint) final;
-    void createSnapshot(OperationContext* txn, SnapshotName name) final;
-    void forceSnapshotCreation() final;
+    void updateCommittedSnapshot(const OpTime& newCommitPoint) final;
     virtual bool snapshotsEnabled() const;
-    virtual void notifyOplogMetadataWaiters();
+    virtual void notifyOplogMetadataWaiters(const OpTime& committedOpTime);
     virtual double getElectionTimeoutOffsetLimitFraction() const;
-    virtual bool isReadCommittedSupportedByStorageEngine(OperationContext* txn) const;
-    virtual StatusWith<OpTime> multiApply(OperationContext* txn,
+    virtual bool isReadCommittedSupportedByStorageEngine(OperationContext* opCtx) const;
+    virtual StatusWith<OpTime> multiApply(OperationContext* opCtx,
                                           MultiApplier::Operations ops,
                                           MultiApplier::ApplyOperationFn applyOperation) override;
     virtual Status multiSyncApply(MultiApplier::OperationPtrs* ops) override;
@@ -113,12 +113,11 @@ public:
                                          const HostAndPort& source,
                                          AtomicUInt32* fetchCount) override;
     virtual std::unique_ptr<OplogBuffer> makeInitialSyncOplogBuffer(
-        OperationContext* txn) const override;
+        OperationContext* opCtx) const override;
     virtual std::unique_ptr<OplogBuffer> makeSteadyStateOplogBuffer(
-        OperationContext* txn) const override;
-    virtual bool shouldUseDataReplicatorInitialSync() const override;
-    virtual std::size_t getOplogFetcherSteadyStateMaxFetcherRestarts() const override;
-    virtual std::size_t getOplogFetcherInitialSyncMaxFetcherRestarts() const override;
+        OperationContext* opCtx) const override;
+    virtual std::size_t getOplogFetcherMaxFetcherRestarts() const override;
+    SyncTail::BatchLimits getInitialSyncBatchLimits() const final;
 
     // Methods from JournalListener.
     virtual JournalListener::Token getToken();
@@ -132,7 +131,7 @@ private:
     /**
      * Stops data replication and returns with 'lock' locked.
      */
-    void _stopDataReplication_inlock(OperationContext* txn, UniqueLock* lock);
+    void _stopDataReplication_inlock(OperationContext* opCtx, UniqueLock* lock);
 
     /**
      * Called when the instance transitions to primary in order to notify a potentially sharded host
@@ -140,15 +139,17 @@ private:
      *
      * Throws on errors.
      */
-    void _shardingOnTransitionToPrimaryHook(OperationContext* txn);
+    void _shardingOnTransitionToPrimaryHook(OperationContext* opCtx);
 
     /**
     * Drops all temporary collections on all databases except "local".
     *
     * The implementation may assume that the caller has acquired the global exclusive lock
-    * for "txn".
+    * for "opCtx".
     */
-    void _dropAllTempCollections(OperationContext* txn);
+    void _dropAllTempCollections(OperationContext* opCtx);
+
+    ServiceContext* _service;
 
     // Guards starting threads and setting _startedThreads
     stdx::mutex _threadMutex;
@@ -157,10 +158,20 @@ private:
     bool _stoppingDataReplication = false;
     stdx::condition_variable _dataReplicationStopped;
 
+    // Used to clean up drop-pending collections with drop optimes before the current replica set
+    // committed OpTime.
+    DropPendingCollectionReaper* _dropPendingCollectionReaper;
+
     StorageInterface* _storageInterface;
+
+    ReplicationProcess* _replicationProcess;
+
     // True when the threads have been started
     bool _startedThreads = false;
 
+    // Set to true when we are in the process of shutting down replication.
+    bool _inShutdown = false;
+
     // The SyncSourceFeedback class is responsible for sending replSetUpdatePosition commands
     // for forwarding replication progress information upstream when there is chained
     // replication.
@@ -184,14 +195,6 @@ private:
     // Number used to uniquely name threads.
     long long _nextThreadId = 0;
 
-    std::unique_ptr<SnapshotThread> _snapshotThread;
-
-    // Initial sync stuff
-    StartInitialSyncFn _startInitialSyncIfNeededFn;
-    StartSteadyReplicationFn _startSteadReplicationFn;
-    OldThreadPool _initialSyncThreadPool;
-    TaskRunner _initialSyncRunner;
-
     // Task executor used to run replication tasks.
     std::unique_ptr<executor::TaskExecutor> _taskExecutor;
 
