@@ -50,39 +50,51 @@
 #include "mongo/db/auth/authz_manager_external_state_s.h"
 #include "mongo/db/auth/user_cache_invalidator_job.h"
 #include "mongo/db/client.h"
-#include "mongo/db/dbwebserver.h"
 #include "mongo/db/ftdc/ftdc_mongos.h"
+#include "mongo/db/generic_cursor_manager.h"
+#include "mongo/db/generic_cursor_manager_mongos.h"
 #include "mongo/db/initialize_server_global_state.h"
-#include "mongo/db/instance.h"
+#include "mongo/db/kill_sessions.h"
 #include "mongo/db/lasterror.h"
 #include "mongo/db/log_process_details.h"
+#include "mongo/db/logical_clock.h"
+#include "mongo/db/logical_session_cache_factory_mongos.h"
+#include "mongo/db/logical_time_metadata_hook.h"
+#include "mongo/db/logical_time_validator.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/server_options.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/service_context_noop.h"
+#include "mongo/db/session_killer.h"
 #include "mongo/db/startup_warnings_common.h"
 #include "mongo/db/wire_version.h"
 #include "mongo/executor/task_executor_pool.h"
 #include "mongo/platform/process_id.h"
+#include "mongo/rpc/metadata/egress_metadata_hook_list.h"
 #include "mongo/s/balancer_configuration.h"
 #include "mongo/s/catalog/sharding_catalog_client.h"
-#include "mongo/s/catalog/sharding_catalog_manager.h"
+#include "mongo/s/catalog_cache.h"
 #include "mongo/s/client/shard_connection.h"
 #include "mongo/s/client/shard_factory.h"
 #include "mongo/s/client/shard_registry.h"
 #include "mongo/s/client/shard_remote.h"
-#include "mongo/s/client/sharding_connection_hook_for_mongos.h"
+#include "mongo/s/client/sharding_connection_hook.h"
+#include "mongo/s/commands/kill_sessions_remote.h"
+#include "mongo/s/config_server_catalog_cache_loader.h"
 #include "mongo/s/grid.h"
+#include "mongo/s/is_mongos.h"
 #include "mongo/s/mongos_options.h"
 #include "mongo/s/query/cluster_cursor_cleanup_job.h"
 #include "mongo/s/query/cluster_cursor_manager.h"
 #include "mongo/s/service_entry_point_mongos.h"
 #include "mongo/s/sharding_egress_metadata_hook_for_mongos.h"
+#include "mongo/s/sharding_egress_metadata_hook_for_mongos.h"
 #include "mongo/s/sharding_initialization.h"
 #include "mongo/s/sharding_uptime_reporter.h"
 #include "mongo/s/version_mongos.h"
 #include "mongo/stdx/memory.h"
 #include "mongo/stdx/thread.h"
-#include "mongo/transport/transport_layer_legacy.h"
+#include "mongo/transport/transport_layer_manager.h"
 #include "mongo/util/admin_access.h"
 #include "mongo/util/cmdline_utils/censor_cmdline.h"
 #include "mongo/util/concurrency/idle_thread_block.h"
@@ -92,15 +104,17 @@
 #include "mongo/util/fast_clock_source_factory.h"
 #include "mongo/util/log.h"
 #include "mongo/util/net/message.h"
+#include "mongo/util/net/sock.h"
 #include "mongo/util/net/socket_exception.h"
 #include "mongo/util/net/ssl_manager.h"
 #include "mongo/util/ntservice.h"
 #include "mongo/util/options_parser/startup_options.h"
+#include "mongo/util/periodic_runner.h"
+#include "mongo/util/periodic_runner_factory.h"
 #include "mongo/util/processinfo.h"
 #include "mongo/util/quick_exit.h"
 #include "mongo/util/signal_handlers.h"
 #include "mongo/util/stacktrace.h"
-#include "mongo/util/static_observer.h"
 #include "mongo/util/stringutils.h"
 #include "mongo/util/system_clock_source.h"
 #include "mongo/util/system_tick_source.h"
@@ -118,6 +132,45 @@ namespace {
 
 boost::optional<ShardingUptimeReporter> shardingUptimeReporter;
 
+static constexpr auto kRetryInterval = Seconds{1};
+
+Status waitForSigningKeys(OperationContext* opCtx) {
+    while (true) {
+        // this should be true when shard registry is up
+        invariant(grid.shardRegistry()->isUp());
+        auto configCS = grid.shardRegistry()->getConfigServerConnectionString();
+        auto rsm = ReplicaSetMonitor::get(configCS.getSetName());
+        // mongod will set minWireVersion == maxWireVersion for isMaster requests from
+        // internalClient.
+        if (rsm && (rsm->getMaxWireVersion() < WireVersion::SUPPORTS_OP_MSG ||
+                    rsm->getMaxWireVersion() != rsm->getMinWireVersion())) {
+            log() << "Not waiting for signing keys, not supported by the config shard "
+                  << configCS.getSetName();
+            return Status::OK();
+        }
+        auto stopStatus = opCtx->checkForInterruptNoAssert();
+        if (!stopStatus.isOK()) {
+            return stopStatus;
+        }
+
+        try {
+            if (LogicalTimeValidator::get(opCtx)->shouldGossipLogicalTime()) {
+                return Status::OK();
+            }
+            log() << "Waiting for signing keys, sleeping for " << kRetryInterval
+                  << " and trying again.";
+            sleepFor(kRetryInterval);
+            continue;
+        } catch (const DBException& ex) {
+            Status status = ex.toStatus();
+            warning() << "Error waiting for signing keys, sleeping for " << kRetryInterval
+                      << " and trying again " << causedBy(status);
+            sleepFor(kRetryInterval);
+            continue;
+        }
+    }
+}
+
 }  // namespace
 
 #if defined(_WIN32)
@@ -126,6 +179,10 @@ ntservice::NtServiceDefaultStrings defaultServiceStrings = {
 static ExitCode initService();
 #endif
 
+#if !defined(__has_feature)
+#define __has_feature(x) 0
+#endif
+
 // NOTE: This function may be called at any time after
 // registerShutdownTask is called below. It must not depend on the
 // prior execution of mongo initializers or the existence of threads.
@@ -135,25 +192,75 @@ static void cleanupTask() {
         Client::initThreadIfNotAlready();
         Client& client = cc();
         ServiceContext::UniqueOperationContext uniqueTxn;
-        OperationContext* txn = client.getOperationContext();
-        if (!txn) {
+        OperationContext* opCtx = client.getOperationContext();
+        if (!opCtx) {
             uniqueTxn = client.makeOperationContext();
-            txn = uniqueTxn.get();
+            opCtx = uniqueTxn.get();
         }
 
-        if (serviceContext)
+        if (serviceContext) {
             serviceContext->setKillAllOperations();
 
-        if (auto cursorManager = Grid::get(txn)->getCursorManager()) {
-            cursorManager->shutdown();
+            // Shut down the background periodic task runner.
+            auto runner = serviceContext->getPeriodicRunner();
+            if (runner) {
+                runner->shutdown();
+            }
         }
-        if (auto pool = Grid::get(txn)->getExecutorPool()) {
+
+        // Perform all shutdown operations after setKillAllOperations is called in order to ensure
+        // that any pending threads are about to terminate
+
+        if (auto validator = LogicalTimeValidator::get(serviceContext)) {
+            validator->shutDown();
+        }
+
+        if (auto cursorManager = Grid::get(opCtx)->getCursorManager()) {
+            cursorManager->shutdown(opCtx);
+        }
+
+        if (auto pool = Grid::get(opCtx)->getExecutorPool()) {
             pool->shutdownAndJoin();
         }
-        if (auto catalog = Grid::get(txn)->catalogClient(txn)) {
-            catalog->shutDown(txn);
+
+        if (auto catalog = Grid::get(opCtx)->catalogClient()) {
+            catalog->shutDown(opCtx);
         }
 
+#if __has_feature(address_sanitizer)
+        // When running under address sanitizer, we get false positive leaks due to disorder around
+        // the lifecycle of a connection and request. When we are running under ASAN, we try a lot
+        // harder to dry up the server from active connections before going on to really shut down.
+
+        // Shutdown the TransportLayer so that new connections aren't accepted
+        if (auto tl = serviceContext->getTransportLayer()) {
+            log(LogComponent::kNetwork)
+                << "shutdown: going to close all sockets because ASAN is active...";
+
+            tl->shutdown();
+        }
+
+        // Shut down the global dbclient pool so callers stop waiting for connections.
+        shardConnectionPool.shutdown();
+
+        // Shutdown the Service Entry Point and its sessions and give it a grace period to complete.
+        if (auto sep = serviceContext->getServiceEntryPoint()) {
+            if (!sep->shutdown(Seconds(10))) {
+                log(LogComponent::kNetwork)
+                    << "Service entry point failed to shutdown within timelimit.";
+            }
+        }
+
+        // Shutdown and wait for the service executor to exit
+        if (auto svcExec = serviceContext->getServiceExecutor()) {
+            Status status = svcExec->shutdown(Seconds(5));
+            if (!status.isOK()) {
+                log(LogComponent::kNetwork)
+                    << "Service executor failed to shutdown within timelimit: " << status.reason();
+            }
+        }
+#endif
+
         // Shutdown Full-Time Data Capture
         stopMongoSFTDC();
     }
@@ -161,21 +268,11 @@ static void cleanupTask() {
     audit::logShutdown(Client::getCurrent());
 }
 
-static BSONObj buildErrReply(const DBException& ex) {
-    BSONObjBuilder errB;
-    errB.append("$err", ex.what());
-    errB.append("code", ex.getCode());
-    if (!ex._shard.empty()) {
-        errB.append("shard", ex._shard);
-    }
-    return errB.obj();
-}
-
 }  // namespace mongo
 
 using namespace mongo;
 
-static Status initializeSharding(OperationContext* txn) {
+static Status initializeSharding(OperationContext* opCtx) {
     auto targeterFactory = stdx::make_unique<RemoteCommandTargeterFactoryImpl>();
     auto targeterFactoryPtr = targeterFactory.get();
 
@@ -199,33 +296,50 @@ static Status initializeSharding(OperationContext* txn) {
     auto shardFactory =
         stdx::make_unique<ShardFactory>(std::move(buildersMap), std::move(targeterFactory));
 
+    CatalogCacheLoader::set(opCtx->getServiceContext(),
+                            stdx::make_unique<ConfigServerCatalogCacheLoader>());
+
     Status status = initializeGlobalShardingState(
-        txn,
+        opCtx,
         mongosGlobalParams.configdbs,
-        generateDistLockProcessId(txn),
+        generateDistLockProcessId(opCtx),
         std::move(shardFactory),
-        []() { return stdx::make_unique<rpc::ShardingEgressMetadataHookForMongos>(); },
-        [](ShardingCatalogClient* catalogClient, std::unique_ptr<executor::TaskExecutor> executor) {
-            return nullptr;  // Only config servers get a real ShardingCatalogManager.
-        });
+        stdx::make_unique<CatalogCache>(CatalogCacheLoader::get(opCtx)),
+        [opCtx]() {
+            auto hookList = stdx::make_unique<rpc::EgressMetadataHookList>();
+            hookList->addHook(
+                stdx::make_unique<rpc::LogicalTimeMetadataHook>(opCtx->getServiceContext()));
+            hookList->addHook(stdx::make_unique<rpc::ShardingEgressMetadataHookForMongos>(
+                opCtx->getServiceContext()));
+            return hookList;
+        },
+        boost::none);
 
     if (!status.isOK()) {
         return status;
     }
 
-    status = reloadShardRegistryUntilSuccess(txn);
+    status = waitForShardRegistryReload(opCtx);
     if (!status.isOK()) {
         return status;
     }
 
+    status = waitForSigningKeys(opCtx);
+    if (!status.isOK()) {
+        return status;
+    }
+
+    Grid::get(opCtx)->setShardingInitialized();
+
     return Status::OK();
 }
 
 static void _initWireSpec() {
     WireSpec& spec = WireSpec::instance();
-    // connect to version supporting Write Concern only
-    spec.outgoing.minWireVersion = COMMANDS_ACCEPT_WRITE_CONCERN;
-    spec.outgoing.maxWireVersion = COMMANDS_ACCEPT_WRITE_CONCERN;
+    // Since the upgrade order calls for upgrading mongos last, it only needs to talk the latest
+    // wire version. This ensures that users will get errors if they upgrade in the wrong order.
+    spec.outgoing.minWireVersion = LATEST_WIRE_VERSION;
+    spec.outgoing.maxWireVersion = LATEST_WIRE_VERSION;
 
     spec.isInternalClient = true;
 }
@@ -236,25 +350,34 @@ static ExitCode runMongosServer() {
 
     _initWireSpec();
 
-    transport::TransportLayerLegacy::Options opts;
-    opts.port = serverGlobalParams.port;
-    opts.ipList = serverGlobalParams.bind_ip;
-
-    auto sep =
-        stdx::make_unique<ServiceEntryPointMongos>(getGlobalServiceContext()->getTransportLayer());
-    auto sepPtr = sep.get();
-
+    auto sep = stdx::make_unique<ServiceEntryPointMongos>(getGlobalServiceContext());
     getGlobalServiceContext()->setServiceEntryPoint(std::move(sep));
 
-    auto transportLayer = stdx::make_unique<transport::TransportLayerLegacy>(opts, sepPtr);
-    auto res = transportLayer->setup();
+    auto tl = transport::TransportLayerManager::createWithConfig(&serverGlobalParams,
+                                                                 getGlobalServiceContext());
+    auto res = tl->setup();
     if (!res.isOK()) {
+        error() << "Failed to set up listener: " << res;
         return EXIT_NET_ERROR;
     }
+    getGlobalServiceContext()->setTransportLayer(std::move(tl));
+
+    auto unshardedHookList = stdx::make_unique<rpc::EgressMetadataHookList>();
+    unshardedHookList->addHook(
+        stdx::make_unique<rpc::LogicalTimeMetadataHook>(getGlobalServiceContext()));
+    unshardedHookList->addHook(
+        stdx::make_unique<rpc::ShardingEgressMetadataHookForMongos>(getGlobalServiceContext()));
 
     // Add sharding hooks to both connection pools - ShardingConnectionHook includes auth hooks
-    globalConnPool.addHook(new ShardingConnectionHookForMongos(false));
-    shardConnectionPool.addHook(new ShardingConnectionHookForMongos(true));
+    globalConnPool.addHook(new ShardingConnectionHook(false, std::move(unshardedHookList)));
+
+    auto shardedHookList = stdx::make_unique<rpc::EgressMetadataHookList>();
+    shardedHookList->addHook(
+        stdx::make_unique<rpc::LogicalTimeMetadataHook>(getGlobalServiceContext()));
+    shardedHookList->addHook(
+        stdx::make_unique<rpc::ShardingEgressMetadataHookForMongos>(getGlobalServiceContext()));
+
+    shardConnectionPool.addHook(new ShardingConnectionHook(true, std::move(shardedHookList)));
 
     ReplicaSetMonitor::setAsynchronousConfigChangeHook(
         &ShardRegistry::replicaSetChangeConfigServerUpdateHook);
@@ -271,11 +394,14 @@ static ExitCode runMongosServer() {
 
     auto opCtx = cc().makeOperationContext();
 
+    auto logicalClock = stdx::make_unique<LogicalClock>(opCtx->getServiceContext());
+    LogicalClock::set(opCtx->getServiceContext(), std::move(logicalClock));
+
     {
         Status status = initializeSharding(opCtx.get());
         if (!status.isOK()) {
             if (status == ErrorCodes::CallbackCanceled) {
-                invariant(inShutdown());
+                invariant(globalInShutdownDeprecated());
                 log() << "Shutdown called before mongos finished starting up";
                 return EXIT_CLEAN;
             }
@@ -283,18 +409,10 @@ static ExitCode runMongosServer() {
             return EXIT_SHARDING_ERROR;
         }
 
-        Grid::get(opCtx.get())->getBalancerConfiguration()->refreshAndCheck(opCtx.get());
-    }
-
-    if (serverGlobalParams.isHttpInterfaceEnabled) {
-        std::shared_ptr<DbWebServer> dbWebServer(new DbWebServer(serverGlobalParams.bind_ip,
-                                                                 serverGlobalParams.port + 1000,
-                                                                 getGlobalServiceContext(),
-                                                                 new NoAdminAccess()));
-        dbWebServer->setupSockets();
-
-        stdx::thread web(stdx::bind(&webServerListenThread, dbWebServer));
-        web.detach();
+        Grid::get(opCtx.get())
+            ->getBalancerConfiguration()
+            ->refreshAndCheck(opCtx.get())
+            .transitional_ignore();
     }
 
     startMongoSFTDC();
@@ -320,11 +438,33 @@ static ExitCode runMongosServer() {
 
     PeriodicTask::startRunningPeriodicTasks();
 
-    auto start = getGlobalServiceContext()->addAndStartTransportLayer(std::move(transportLayer));
+    // Set up the periodic runner for background job execution
+    auto runner = makePeriodicRunner();
+    runner->startup().transitional_ignore();
+    getGlobalServiceContext()->setPeriodicRunner(std::move(runner));
+
+    SessionKiller::set(
+        getGlobalServiceContext(),
+        std::make_shared<SessionKiller>(getGlobalServiceContext(), killSessionsRemote));
+
+    GenericCursorManager::set(getGlobalServiceContext(),
+                              stdx::make_unique<GenericCursorManagerMongos>());
+
+    // Set up the logical session cache
+    LogicalSessionCache::set(getGlobalServiceContext(), makeLogicalSessionCacheS());
+
+    auto start = getGlobalServiceContext()->getServiceExecutor()->start();
+    if (!start.isOK()) {
+        error() << "Failed to start the service executor: " << start;
+        return EXIT_NET_ERROR;
+    }
+
+    start = getGlobalServiceContext()->getTransportLayer()->start();
     if (!start.isOK()) {
         return EXIT_NET_ERROR;
     }
 
+    getGlobalServiceContext()->notifyStartupComplete();
 #if !defined(_WIN32)
     mongo::signalForkSuccess();
 #else
@@ -339,18 +479,20 @@ static ExitCode runMongosServer() {
     return waitForShutdown();
 }
 
+namespace {
 MONGO_INITIALIZER_GENERAL(ForkServer, ("EndStartupOptionHandling"), ("default"))
 (InitializerContext* context) {
     mongo::forkServerOrDie();
     return Status::OK();
 }
+}  // namespace
 
-// We set the featureCompatibilityVersion to 3.4 in the mongos so that BSON validation always uses
-// BSONVersion::kLatest.
-MONGO_INITIALIZER_WITH_PREREQUISITES(SetFeatureCompatibilityVersion34, ("EndStartupOptionStorage"))
+// We set the featureCompatibilityVersion to 3.6 in the mongos and rely on the shards to reject
+// usages of new features if their featureCompatibilityVersion is lower.
+MONGO_INITIALIZER_WITH_PREREQUISITES(SetFeatureCompatibilityVersion36, ("EndStartupOptionStorage"))
 (InitializerContext* context) {
-    mongo::serverGlobalParams.featureCompatibility.version.store(
-        ServerGlobalParams::FeatureCompatibility::Version::k34);
+    mongo::serverGlobalParams.featureCompatibility.setVersion(
+        ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo36);
     return Status::OK();
 }
 
@@ -415,6 +557,7 @@ static ExitCode initService() {
 #endif
 
 namespace {
+
 std::unique_ptr<AuthzManagerExternalState> createAuthzManagerExternalStateMongos() {
     return stdx::make_unique<AuthzManagerExternalStateMongos>();
 }
@@ -441,7 +584,8 @@ MONGO_INITIALIZER_GENERAL(setSSLManagerType, MONGO_NO_PREREQUISITES, ("SSLManage
 #endif
 
 int mongoSMain(int argc, char* argv[], char** envp) {
-    static StaticObserver staticObserver;
+    mongo::setMongos();
+
     if (argc < 1)
         return EXIT_FAILURE;
 
